@@ -24,6 +24,8 @@ import regex as re
 
 from arelle import Locale, ModelManager, PackageManager, PluginManager, XbrlConst
 from arelle.BetaFeatures import BETA_FEATURES_AND_DESCRIPTIONS
+from arelle.ErrorManager import ErrorManager
+from arelle.FileSource import FileSource
 from arelle.SystemInfo import PlatformOS, getSystemWordSize, hasFileSystem, hasWebServer, isCGI, isGAE
 from arelle.WebCache import WebCache
 from arelle.logging.formatters.LogFormatter import LogFormatter, logRefsFileLines  # noqa: F401 - for reimport
@@ -123,6 +125,7 @@ class Cntlr:
     """
     __version__ = "1.6.0"
     betaFeatures: dict[str, bool]
+    errorManager: ErrorManager | None
     hasWin32gui: bool
     hasGui: bool
     hasFileSystem: bool
@@ -164,6 +167,7 @@ class Cntlr:
             b: betaFeatures.get(b, False)
             for b in BETA_FEATURES_AND_DESCRIPTIONS.keys()
         }
+        self.errorManager = None
         self.hasWin32gui = False
         self.hasGui = hasGui
         self.hasFileSystem = hasFileSystem() # no file system on Google App Engine servers
@@ -291,6 +295,32 @@ class Cntlr:
         PackageManager.init(self, loadPackagesConfig=hasGui)
 
         self.startLogging(logFileName, logFileMode, logFileEncoding, logFormat)
+        self.errorManager = ErrorManager(self.modelManager, logging._checkLevel("INCONSISTENCY")) # type: ignore[attr-defined]
+
+    def error(self, codes: Any, msg: str, level: str = "ERROR", fileSource: FileSource | None = None, **args: Any) -> None:
+        if self.logger is None or self.errorManager is None:
+            self.addToLog(
+                message=msg,
+                messageCode=str(codes),
+                messageArgs=args,
+                level=level
+            )
+            return
+        self.errorManager.log(
+            self.logger,
+            level,
+            codes,
+            msg,
+            fileSource=fileSource,
+            logRefObjectProperties=getattr(self.logger, "logRefObjectProperties", False),
+            **args
+        )
+
+    @property
+    def errors(self) -> list[str | None]:
+        if self.errorManager is None:
+            return []
+        return self.errorManager.errors
 
     @property
     def uiLangDir(self) -> str:
@@ -662,3 +692,8 @@ class Cntlr:
 
     def _clearPluginData(self) -> None:
         self.__pluginData.clear()
+
+    def testcaseVariationReset(self) -> None:
+        self._clearPluginData()
+        if self.errorManager is not None:
+            self.errorManager.clear()
