@@ -298,6 +298,7 @@ class Validate:
             self._testcaseValidateInputDTS(testcase, modelTestcaseVariation, errorCaptureLevel, parameters, inputDTSes, baseForElement, resultIsXbrlInstance)
         # update ui thread via modelManager (running in background here)
         self.modelXbrl.modelManager.viewModelObject(self.modelXbrl, modelTestcaseVariation.objectId())
+        self.modelXbrl.modelManager.cntlr.testcaseVariationReset()
         modelTestcaseVariation.duration = time.perf_counter() - startTime
 
     def _testcaseLoadReadMeFirstUri(self, testcase, modelTestcaseVariation, index, readMeFirstUri, resultIsVersioningReport, resultIsTaxonomyPackage, inputDTSes, errorCaptureLevel, baseForElement, parameters):
@@ -730,6 +731,7 @@ class Validate:
             _errors = [e for e in errors if isinstance(e,str) and not _blockPattern.match(e)]
         else:
             _errors = errors
+        _errors.extend(self.modelXbrl.modelManager.cntlr.errors)
         numErrors = sum(isinstance(e,(QName,str)) for e in _errors) # does not include asserton dict results
         hasAssertionResult = any(isinstance(e,dict) for e in _errors)
         expected = modelTestcaseVariation.expected
@@ -764,7 +766,7 @@ class Validate:
             status = "pass" if numErrors == 0 else "fail"
         elif expected == "invalid":
             status = "fail" if numErrors == 0 else "pass"
-        elif expected in (None, []) and numErrors == 0:
+        elif expected in (None, []) and numErrors == 0 and not expectedWarnings:
             status = "pass"
         elif isinstance(expected, (QName, str, dict, list)) or expectedWarnings:
             status = "fail"
