@@ -5,6 +5,7 @@ from __future__ import annotations
 
 import datetime
 import itertools
+from collections import defaultdict
 from collections.abc import Callable, Iterable
 from typing import Optional, cast
 
@@ -14,6 +15,7 @@ from arelle.ModelValue import QName
 from arelle.ModelXbrl import ModelXbrl
 from arelle.typing import TypeGetText
 from arelle.UrlUtil import scheme
+from arelle.utils.Contexts import ContextHashKey
 from arelle.utils.validate.Validation import Validation
 from arelle.ValidateFilingText import parseImageDataURL
 from arelle.ValidateXbrl import ValidateXbrl
@@ -266,19 +268,18 @@ def getFactsGroupedByContextId(modelXbrl: ModelXbrl, *conceptQns: QName) -> dict
     return dict(sorted(groupedFacts.items()))
 
 
-def groupFactsByContextHash(facts: set[ModelFact] ) -> dict[str, list[ModelFact]]:
+def groupFactsByContextHash(facts: set[ModelFact]) -> dict[ContextHashKey, list[ModelFact]]:
     """
-    Groups facts by their contextDimAwareHash.
-    :return: A dictionary of contextDimAwareHashes to list of facts.
+    Groups facts by their context hash key.
+    :return: A dictionary of context hash keys to list of facts.
     """
-    groupedFacts: dict[str, list[ModelFact]] = {}
+    groupedFacts: defaultdict[ContextHashKey, list[ModelFact]] = defaultdict(list)
     for fact in facts:
         if fact.xValid >= VALID:
-            contextHash = fact.context.contextDimAwareHash
-            if contextHash not in groupedFacts:
-                groupedFacts[contextHash] = []
+            contextHash = ContextHashKey(fact.context, dimensionalAspectModel=True)
             groupedFacts[contextHash].append(fact)
-    return dict(sorted(groupedFacts.items()))
+    groupedFacts.default_factory = None
+    return groupedFacts
 
 
 def lookup_namespaced_facts(modelXbrl: ModelXbrl, namespaceURI: str) -> set[ModelFact]:
