@@ -14,6 +14,7 @@ from lxml.etree import _Comment, _ElementTree, _Entity, _ProcessingInstruction,
 
 from arelle import XbrlConst
 from arelle.FunctionIxt import ixtNamespaces
+from arelle.LinkbaseType import LinkbaseType
 from arelle.ModelDocument import ModelDocument, Type as ModelDocumentType
 from arelle.ModelDtsObject import ModelConcept, ModelRelationship
 from arelle.ModelInstanceObject import ModelContext, ModelFact, ModelInlineFootnote, ModelUnit, ModelInlineFact
@@ -27,7 +28,6 @@ from arelle.utils.validate.ValidationUtil import etreeIterWithDepth
 from arelle.XbrlConst import ixbrl11, xhtmlBaseIdentifier, xmlBaseIdentifier
 from arelle.XmlValidate import lexicalPatterns
 from arelle.XmlValidateConst import VALID
-from .LinkbaseType import LinkbaseType
 
 DEFAULT_MEMBER_ROLE_URI = 'https://www.nltaxonomie.nl/kvk/role/axis-defaults'
 XBRLI_IDENTIFIER_PATTERN = re.compile(r"^(?!00)\d{8}$")
@@ -225,7 +225,7 @@ class PluginValidationDataExtension(PluginData):
     financialReportingPeriodCurrentEndDateQn: QName
     financialReportingPeriodPreviousStartDateQn: QName
     financialReportingPeriodPreviousEndDateQn: QName
-    formattedExplanationItemTypeQn: QName
+    formattedExplanationItemTypeQn: QName | None
     ifrsIdentifier: str
     permissibleGAAPRootAbstracts: frozenset[QName]
     permissibleIFRSRootAbstracts: frozenset[QName]
@@ -242,17 +242,10 @@ class PluginValidationDataExtension(PluginData):
             for domMbrRel in modelXbrl.relationshipSet(XbrlConst.domainMember, ELR).fromModelObject(sourceDomMbr):
                 self.addDomMbrs(modelXbrl, domMbrRel.toModelObject, domMbrRel.consecutiveLinkrole, membersSet)
 
-    @lru_cache(1)
-    def contextsByDocument(self, modelXbrl: ModelXbrl) -> dict[str, list[ModelContext]]:
-        contextsByDocument = defaultdict(list)
-        for context in modelXbrl.contexts.values():
-            contextsByDocument[context.modelDocument.filepath].append(context)
-        contextsByDocument.default_factory = None
-        return contextsByDocument
 
     @lru_cache(1)
     def checkContexts(self, modelXbrl: ModelXbrl) -> ContextData:
-        allContexts = self.contextsByDocument(modelXbrl)
+        allContexts = modelXbrl.contextsByDocument()
         contextsWithImproperContent: list[ModelContext | None] = []
         contextsWithPeriodTime: list[ModelContext | None] = []
         contextsWithPeriodTimeZone: list[ModelContext | None] = []
@@ -395,9 +388,6 @@ class PluginValidationDataExtension(PluginData):
             tupleElements=tupleElements,
         )
 
-    @lru_cache(1)
-    def entityIdentifiersInDocument(self, modelXbrl: ModelXbrl) -> set[tuple[str, str]]:
-        return {context.entityIdentifier for context in modelXbrl.contexts.values()}
 
     @lru_cache(1)
     def factsByDocument(self, modelXbrl: ModelXbrl) -> dict[str, list[ModelFact]]:
@@ -424,25 +414,13 @@ class PluginValidationDataExtension(PluginData):
         calcRelSet = modelXbrl.relationshipSet(XbrlConst.summationItems)
         dimensionalData = self.getDimensionalData(modelXbrl)
         primaryItems = dimensionalData.primaryItems
-        domainMembers = dimensionalData.domainMembers
         extensionData = self.getExtensionData(modelXbrl)
         for concept in extensionData.extensionConcepts:
-            extLineItem = False
             if concept.isPrimaryItem and \
                     not concept.isAbstract and \
                     concept in primaryItems and \
                     not widerNarrowerRelSet.contains(concept) and \
                     not calcRelSet.fromModelObject(concept):
-                extLineItem = True
-            elif concept.isAbstract and \
-                    concept not in domainMembers and \
-                    concept.type is not None and \
-                    not concept.type.isDomainItemType and \
-                    not concept.isHypercubeItem and \
-                    not concept.isDimensionItem and \
-                    not widerNarrowerRelSet.contains(concept):
-                extLineItem = True
-            if extLineItem:
                 if not generalSpecialRelSet.contains(concept):
                     extLineItemsNotAnchored.add(concept)
                 else:
