@@ -4,10 +4,11 @@ See COPYRIGHT.md for copyright information.
 from __future__ import annotations
 
 import os
-from typing import Any
+from typing import Any, cast
 
 from collections import Counter
 from collections.abc import Iterable
+from decimal import Decimal
 
 from arelle.typing import TypeGetText
 from arelle.ValidateXbrl import ValidateXbrl
@@ -19,14 +20,15 @@ from arelle.ModelInstanceObject import ModelInlineFact, ModelUnit
 from arelle.ModelValue import qname
 from arelle.ModelXbrl import ModelXbrl
 from arelle.PythonUtil import strTruncate
+from arelle.utils.Contexts import partitionModelXbrlContexts
+from arelle.utils.Units import partitionModelXbrlUnits
 from arelle.utils.PluginHooks import ValidationHook
 from arelle.utils.validate.Decorator import validation
 from arelle.utils.validate.Validation import Validation
 from arelle.ValidateXbrlCalcs import inferredDecimals, rangeValue
 from arelle.XbrlConst import qnXbrliMonetaryItemType, qnXbrliXbrl, xhtml
 from arelle.XmlValidateConst import VALID
-from . import errorOnNegativeFact
-from ..ValidationPluginExtension import IE_PROFIT_LOSS_ORDINARY, IE_PROFIT_LOSS, PRINCIPAL_CURRENCY, TURNOVER_REVENUE
+from ..ValidationPluginExtension import EQUITY, PRINCIPAL_CURRENCY, TURNOVER_REVENUE
 from ..PluginValidationDataExtension import MANDATORY_ELEMENTS,  SCHEMA_PATTERNS, TR_NAMESPACES, PluginValidationDataExtension
 
 
@@ -167,7 +169,6 @@ def rule_main(
         schemeEntityIds = set()
         mapContext = {} # identify unique contexts and units
         mapUnit = {}
-        uniqueContextHashes: dict[str, str] = {}
         hasCRO = False
         unsupportedSchemeContexts = []
         mismatchIdentifierContexts = []
@@ -180,13 +181,10 @@ def rule_main(
                 mismatchIdentifierContexts.append(context)
             if scheme == "http://www.cro.ie/":
                 hasCRO = True
-            h = context.contextDimAwareHash
-            if h in uniqueContextHashes:
-                if context.isEqualTo(uniqueContextHashes[h]):
-                    mapContext[context] = uniqueContextHashes[h]
-            else:
-                uniqueContextHashes[h] = context
-        del uniqueContextHashes
+        for exemplar_context, *contexts in partitionModelXbrlContexts(modelXbrl).values():
+            for context in contexts:
+                mapContext[context] = exemplar_context
+
         if len(schemeEntityIds) > 1:
             modelXbrl.error("ROS:differentContextEntityIdentifiers",
                             _("Context entity identifier not all the same: %(schemeEntityIds)s."),
@@ -202,15 +200,9 @@ def rule_main(
                             modelObject=mismatchIdentifierContexts,
                             identifiers=", ".join(sorted(set(c.entityIdentifier[1] for c in mismatchIdentifierContexts))))
 
-        uniqueUnitHashes: dict[str, ModelUnit] = {}
-        for unit in modelXbrl.units.values():
-            h = unit.hash
-            if h in uniqueUnitHashes:
-                if unit.isEqualTo(uniqueUnitHashes[h]):
-                    mapUnit[unit] = uniqueUnitHashes[h]
-            else:
-                uniqueUnitHashes[h] = unit
-        del uniqueUnitHashes
+        for exemplar_unit, *units in partitionModelXbrlUnits(modelXbrl).values():
+            for unit in units:
+                mapUnit[unit] = exemplar_unit
 
         if hasCRO and "ie-common" in nsMap:
             mandatory.add(qname("ie-common:CompaniesRegistrationOfficeNumber", nsMap))  # type-ignore[arg-type]
@@ -293,52 +285,6 @@ def rule_main(
     modelXbrl.modelManager.showStatus(None)
 
 
-@validation(
-    hook=ValidationHook.XBRL_FINALLY,
-)
-def rule_ros6(
-        pluginData: PluginValidationDataExtension,
-        val: ValidateXbrl,
-        *args: Any,
-        **kwargs: Any,
-) -> Iterable[Validation]:
-    """
-    ROS: Rule 6: ProfitLossOnOrdinaryActivitiesBeforeTax OR ProfitLossBeforeTax must exist in the document and be non-nil.
-    """
-    non_nil_profit_loss_fact_exists = any(
-        not fact.isNil for fact in val.modelXbrl.factsByLocalName.get(IE_PROFIT_LOSS, set())
-    )
-    non_nil_profit_loss_ordinary_fact_exists = any(
-        not fact.isNil for fact in val.modelXbrl.factsByLocalName.get(IE_PROFIT_LOSS_ORDINARY, set())
-    )
-    if not non_nil_profit_loss_fact_exists and not non_nil_profit_loss_ordinary_fact_exists:
-        yield Validation.error(
-            codes='ROS.6',
-            msg=_("Profit or Loss Before Tax (uk gaap:ProfitLossOnOrdinaryActivitiesBeforeTax) is missing."),
-            modelObject=val.modelXbrl
-        )
-
-
-@validation(
-        hook=ValidationHook.XBRL_FINALLY,
-    )
-def rule_ros18(
-        pluginData: PluginValidationDataExtension,
-        val: ValidateXbrl,
-        *args: Any,
-        **kwargs: Any,
-) -> Iterable[Validation]:
-    """
-    ROS: Rule 18: DPLTurnoverRevenue cannot be a negative value.
-    """
-    return errorOnNegativeFact(
-            val.modelXbrl,
-            conceptLn=TURNOVER_REVENUE,
-            code='ROS.18',
-            message=_("Turnover / Revenue cannot be a negative value.")
-        )
-
-
 @validation(
     hook=ValidationHook.XBRL_FINALLY,
 )
@@ -349,14 +295,52 @@ def rule_ros19(
         **kwargs: Any,
 ) -> Iterable[Validation]:
     """
-    ROS: Rule 19:PrincipalCurrencyUsedInBusinessReport must exist and its value must match the unit
+    ROS: Rule 19: DPLTurnoverRevenue should be tested to be less than or equal to 10x the absolute value of Equity
+    """
+    def convertToDecimal(value):
+        return Decimal(value) if isinstance(value, float) else cast(Decimal, value)
+    equity_facts = val.modelXbrl.factsByLocalName.get(EQUITY, set())
+    largest_equity_fact = None
+    for e_fact in equity_facts:
+        if e_fact.xValid >= VALID:
+            if largest_equity_fact is None or abs(convertToDecimal(e_fact.xValue)) > abs(convertToDecimal(largest_equity_fact.xValue)):
+                largest_equity_fact = e_fact
+
+    turnover_facts = val.modelXbrl.factsByLocalName.get(TURNOVER_REVENUE, set())
+    largest_turnover_fact = None
+    for t_fact in turnover_facts:
+        if t_fact.xValid >= VALID:
+            if largest_turnover_fact is None or convertToDecimal(t_fact.xValue) > convertToDecimal(largest_turnover_fact.xValue):
+                largest_turnover_fact = t_fact
+
+    if (largest_equity_fact is not None and
+            largest_turnover_fact is not None and
+            convertToDecimal(largest_turnover_fact.xValue) > (10 * abs(convertToDecimal(largest_equity_fact.xValue)))):
+        yield Validation.error(
+            "ROS.19",
+            _("Turnover / Revenue (DPLTurnoverRevenue) may exceed the maximum expected value. Please review the submission and, if correct, test your submission with Revenue Online's iXBRL test facility."),
+            modelObject=[largest_equity_fact, largest_turnover_fact],
+        )
+
+
+@validation(
+    hook=ValidationHook.XBRL_FINALLY,
+)
+def rule_ros20(
+        pluginData: PluginValidationDataExtension,
+        val: ValidateXbrl,
+        *args: Any,
+        **kwargs: Any,
+) -> Iterable[Validation]:
+    """
+    ROS: Rule 20:PrincipalCurrencyUsedInBusinessReport must exist and its value must match the unit
     used for the majority of monetary facts.
     """
     principal_currency_facts = val.modelXbrl.factsByLocalName.get(PRINCIPAL_CURRENCY, set())
     principal_currency_values = set(fact.text for fact in principal_currency_facts)
     if len(principal_currency_values) != 1:
         yield Validation.error(
-            "ROS.19",
+            "ROS.20",
             _("'PrincipalCurrencyUsedInBusinessReport' must exist and have a single value.  Values found: %(principal_currency_values)s."),
             modelObject=principal_currency_facts,
             principal_currency_values=principal_currency_values,
@@ -370,7 +354,7 @@ def rule_ros19(
         for unit, count in unit_counts.items():
             if count > principal_currency_value_count:
                 yield Validation.warning(
-                    "ROS.19",
+                    "ROS.20",
                     _("'PrincipalCurrencyUsedInBusinessReport' has a value of %(principal_currency_value)s, "
                       "which must match the functional(majority) unit of the financial statement."),
                     modelObject=principal_currency_facts,
